#!/bin/perl
################################################################################
# Created       : Martin Foster
# Modified      : 07/11/2002
################################################################################
#
# Config Realm - Script part of Webchat designed for supervisors to control realms
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204, 817 5TH ST NE
#                 Calgary, Alberta
#                 T2E 3W9
#
################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Audit;						# Auditing information
use Ethereal::Comm;						# Comm panel
use Ethereal::Database;						# Database handler
use Ethereal::Date;						# Date handler
use Ethereal::Login;						# Login functionality
use Ethereal::Mail;						# Mail handler
use Ethereal::Option;						# Option handler
use Ethereal::Param;						# Parameter control
use Ethereal::Pull;						# Post retrieval
use Ethereal::Table;						# Table handler
use Ethereal::Template;						# Template handler
use Ethereal::Weather;						# Weather handler


################################################################################
# Data Members
################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle
my $pull;							# Post retreival
my $table;							# Table handle
my $tmpl;							# Template handle

my $admin;							# Administrative override
my $access;							# Realm access
my $url;							# Self referencing link
my $verdict;							# Verdict on bans

my %args;							# Default arguments
my %config;							# Configuration hash


################################################################################
# Program Area
################################################################################

	# Initial handle
	$cgi      = new CGI;	
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();
	$pull     = new Ethereal::Pull();

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);

	# Fetch user
	$url = $cgi->url();

	# Fetch hash
	$database->GetHashConfiguration(\%config);

	$table    = new Ethereal::Table($cgi);
	$tmpl     = new Ethereal::Template(\%config);


	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();


	# Login handle
	$login = new Ethereal::Login($database, $cgi, $param);

	# Authenthication
	if ($login->GetVerificationSuper())
	{
		# Display title
		print "$config{'TagTitle'}\n<BR>\n";

		# Fetch Posts
		$pull->AdminPosts($database, $cgi, $param);

		# Realm selection
		unless (defined($param->{'ROOM'}))
		{ 
			RealmSelect($database, $cgi, $param, $table, $tmpl, \%config); 
		}

		# Realm handling
		else
		{ 
			# Prevent bans from use
			($access)  = $database->DataGetRealmAccess($param->{'ROOM'});
			($verdict) = $database->DataGetBannedStatus($param->{'USER'}, $param->{'ROOM'});

			# Administrative access
			($admin)   = $database->DataGetAdmin($param->{'USER'}); 


			# Only show if not banned
			unless (((($access eq 'relaxed') && (defined($verdict)))
			      or (($access eq 'restricted') && (defined($verdict)))
			      or (($access eq 'list') && (!defined($verdict))))
			     and ($admin eq 'no'))
			{
				# Generate arguments
				%args = (
					USER  => $param->{'USER'},
					CRYPT => $param->{'CRYPT'}
				 );


				# Generate inlines
				# Back button
				my ($iback) = $param->EmbedInline($cgi, %args); 


				# Add parameter
				$args{'ROOM'} = $param->{'ROOM'};

				# Regular inlines
				my ($ibann) = $param->EmbedInline($cgi, %args, 'BANN', 'True');
				my ($icnon) = $param->EmbedInline($cgi, %args, 'CANON', 'True');
				my ($idsgn) = $param->EmbedInline($cgi, %args, 'DESIGN', 'True');
				my ($ifntc) = $param->EmbedInline($cgi, %args, 'FONT', 'True');
				my ($iforc) = $param->EmbedInline($cgi, %args, 'FORCAST', 'True');
				my ($ilocl) = $param->EmbedInline($cgi, %args, 'LOCALE', 'True');
				my ($ipics) = $param->EmbedInline($cgi, %args, 'GRAPH', 'True');
				my ($imods) = $param->EmbedInline($cgi, %args, 'MODS', 'True');
				my ($irelm) = $param->EmbedInline($cgi, %args);
				my ($itopc) = $param->EmbedInline($cgi, %args, 'TOPIC', 'True');
				my ($iword) = $param->EmbedInline($cgi, %args, 'WORD', 'True');
				my ($ipost) = $param->EmbedInline($cgi, %args, 'GETPOSTS', 'True');


				# Display URLs
			 	print "<CENTER><A HREF=\"$url$iback\">$config{TagBack}</A> ",
				      "<A HREF=\"$url$irelm\">$config{TagRealm}</A> ",
				      "<A HREF=\"$url$ibann\">$config{TagBann}</A> ",
				      "<A HREF=\"$url$itopc\">$config{TagTopic}</A> ",
				      "<A HREF=\"$url$icnon\">$config{TagCanon}</A> ",
				      "<A HREF=\"$url$idsgn\">$config{TagDesign}</A> ",
				      "<A HREF=\"$url$ifntc\">$config{TagFont}</A> ",
				      "<A HREF=\"$url$iforc\">$config{TagForcast}</A> ",
				      "<A HREF=\"$url$ilocl\">$config{TagLocale}</A> ",
				      "<A HREF=\"$url$imods\">$config{TagMods}</A> ",
				      "<A HREF=\"$url$ipics\">$config{TagPics}</A> ",
				      "<A HREF=\"$url$ipost\">$config{TagPosts}</A> ",
				      "<A HREF=\"$url$iword\">$config{TagWord}</A></CENTER>\n<BR><BR>\n";


				# Realm information
				RoomWrite($database, $cgi, $param, \%config, $access, $admin);

				# Bannings
				if (defined($param->{'BANN'}))
				{
					RoomBan($database, $cgi, $param, $table, $tmpl, \%config);	
				}


				# Canon handles
				elsif (defined($param->{'CANON'}))
				{
					RoomCanon($database, $cgi, $param, $table, $tmpl, \%config);
				}

				# Realm Design
				elsif (defined($param->{'DESIGN'}))
				{
					RoomDesign($database, $cgi, $param, $table, $tmpl, \%config);
				}

				# Font handler
				elsif (defined($param->{'FONT'}))
				{
					FontHandler($database, $cgi, $param, $tmpl, \%config);	
				}

				# Locale handler
				elsif (defined($param->{'LOCALE'}))
				{
					LocaleHandler($database, $cgi, $param, $table, $tmpl, \%config);	
				}


				# Weather Forcasts
				elsif (defined($param->{'FORCAST'}))
				{
					Forcasts($database, $cgi, $param, $table, $tmpl, \%config);
				}

				# Picture Restriction
				elsif (defined($param->{'GRAPH'}))
				{
					RoomPicture($database, $cgi, $param, $table, $tmpl, \%config);
				}

				# Moderator Handling
				elsif (defined($param->{'MODS'}))
				{
					RoomMods($database, $cgi, $param, $table, $tmpl, \%config);	
				}
			

				# Board Topics
				elsif (defined($param->{'TOPIC'}))
				{
					BoardTopic($database, $cgi, $param, $table, $tmpl, \%config);
				}

				# Word Restrictions
				elsif (defined($param->{'WORD'}))
				{
					RoomWord($database, $cgi, $param, $table, $tmpl, \%config);	
				}

				# Realm information
				else
				{
					RoomView($database, $cgi, $param, $table, $tmpl, \%config, $admin);	
				}
			}
		}
	}

	$database->DocumentGetFooter();


################################################################################
# Sub-Routines
################################################################################

#####################
# Message Board Topic
#
# Allows for the addition and removal of topics in relation to their use in
# the message boards integrated with the system itself.

sub BoardTopic
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $table    = shift;					# Table handle
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash

	my $topics;						# List of topics
	my @topics;						# List of topics	


	#####################
	# program area

	# Retreive list of banned and suspects
	$database->GetListTopics(\@topics, $param->{'ROOM'});
	unshift(@topics, $config->{'TxtDefault'});

	# Create widget
	$topics = $cgi->scrolling_list('TOPICREM',  \@topics, $topics[0], 10);


	# HTML
	# Title
	$tmpl->ShowMain($config->{'TagTopcTitle'});

	# Beginning of form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'TOPICS', 'True'), "\n";

	# Table Top
	$table->MakeTop();

	# Remove
	$table->MakeSingle($tmpl->PassSub($config->{'TagTopcRem'}));
	$table->MakeValid($topics, $cgi->submit($config->{'TxtSubmitDelete'}), 'middle');
	$table->MakeBlank();

	# Addition
	$table->MakeSingle($tmpl->PassSub($config->{'TagTopcAdd'}));
	$table->MakeValid($cgi->textfield('TOPICADD', '', 15, 25), $cgi->submit($config->{'TxtSubmitAddition'}));
	$table->MakeBottom();

	# End of form
	print $cgi->end_form(), "\n";
}

#####################
# Font Finder
#
# Makes the ammusing task of determining exactly what to do with what is has
# been given.  If to add or remove the value from the list et cetera.

sub FontFinder
{
	#####################
	# Data members
	my $cgi    = shift;					# CGI handle
	my $param  = shift;					# Parameter handle
	my $array  = shift;					# Array to embed
	my $realm  = shift;					# Realms information 
	my $name   = shift;					# Compare value

	my $value;						# Discovered value
	my @realm;						# Realms choices


	#####################
	# Data members

	# Initial display
	unless (defined($param->{'FONTCTRL'}))
	{
		# Only perform if one can compare
		if (defined($realm))
		{
			# Seperate into useable objects
			@realm = split(/:/, $realm);
	
			# Search
			foreach $value (@realm)
			{		
				# If matched
				if ($value eq $name)
				{
					# Remove space
					$name =~ s/\s//gs;

					# For checked
					return $cgi->checkbox(-name=>$name, -label=>'', -checked=>'ON');
				}
			}
		}

		# Nothing defined assume defaults
		else
		{
			# Remove spaces
			$name =~ s/\s//gs;

			# For checked
			return $cgi->checkbox(-name=>$name, -label=>'', -checked=>'ON');
		}
	}

	# Parameter level handling
	else
	{
		# Assign value
		$value = $name;

		# Remove spaces
		$name =~ s/\s//gs;

		
		# Array handling
		if (defined($param->{$name}))
		{
			# Push onto array
			push(@{$array}, $value);

			# For checked
			return $cgi->checkbox(-name=>$name, -label=>'', -checked=>'ON');
		}
	} 

	# Return false
	return $cgi->checkbox(-name=>$name, -label=>'');
}

#####################
# Font Handler
#
# Mostly based on the same code as the administrative script, this one will 
# allow hosts to actually make a selection of which they wish to allow for
# use.

sub FontHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter reference
	my $tmpl     = shift;					# Font handler
	my $config   = shift;					# Config hash

	my $data     = $database->{'HANDLE'};			# Database handle
	my $res;						# Results			
	my $statement;						# Database statement

	my $ctype;						# Current working type
	my $name;						# Font name
	my $type;						# Font type

	my $rcolor;						# Realms font color
	my $rface;						# Realms font face
	my $rsize;						# Realms font size
	
	my @rcolor;						# Realms font color
	my @rface;						# Realms font face
	my @rsize;						# Realms font size


	#####################
	# Program area

	# Title
	$tmpl->ShowMain($config->{'TagFontTitle'});


	# Retreive realms choices
	($rcolor, $rface, $rsize) = $database->DataGetRealmFont($param->{'ROOM'});	


	# Query management
	# Prepare and execute
	$statement = $data->prepare("SELECT 
		 FontName,
		 FontType,
		 FontCode
		FROM  Font
		ORDER BY FontType, FontCode");
	$statement->execute();


	# Form handle
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, 'USER', $param->{'USER'}, 'CRYPT',  $param->{'CRYPT'}, 'ROOM', $param->{'ROOM'}, 'FONT', 'True', 'FONTCTRL', 'True');

	# HTML display
	# Table Top
	print "<CENTER>\n<TABLE WIDTH=500 CELLPADDING=5>\n";

	# Cycle through fonts
	while ($res = $statement->fetchrow_hashref())
	{
		# Title
		if ((!defined($type))
		 or ($type ne $res->{'FontType'}))
		{
			# Uppercase
			$type = uc($res->{'FontType'});


			# Display row
			print " <TR>\n",
			      "  <TD COLSPAN=3><BR>\n<BR>\n<B><FONT SIZE=+2>$type</FONT></B><BR></TD>\n",
			      " </TR>\n";


			# Alternate for next pass
			$type = $res->{'FontType'};
		}

		# Name handling
		$name = $res->{FontName};

		# Display row
		# Font colour
		if ($type eq 'color')
		{
			# Display colour
			print " <TR>\n",
			      "  <TD><B>$res->{FontName}</B></TD>\n",
			      "  <TD BGCOLOR=\"$res->{FontCode}\" WIDTH=15>&nbsp;</TD>\n",
			      "  <TD><CENTER>", FontFinder($cgi, $param, \@rcolor, $rcolor, $name), "</CENTER></TD>\n",
			      " </TR>\n";
		}

		# Font face
		elsif ($type eq 'face')
		{
			# Current working type
			$ctype = uc($type);

			# Simply show a blank space
			print " <TR>\n",
			      "  <TD><FONT $ctype=\"$res->{FontCode}\">$res->{FontName}</FONT></TD>\n",
			      "  <TD>&nbsp;</TD>\n",
			      "  <TD><CENTER>", FontFinder($cgi, $param, \@rface, $rface, $name), "</CENTER></TD>\n",
			      " </TR>\n";
		}

		# Font size
		elsif ($type eq 'size')
		{
			# Current working type
			$ctype = uc($type);

			# Simply show a blank space
			print " <TR>\n",
			      "  <TD><FONT $ctype=\"$res->{FontCode}\">$res->{FontName}</FONT></TD>\n",
			      "  <TD>&nbsp;</TD>\n",
			      "  <TD><CENTER>", FontFinder($cgi, $param, \@rsize, $rsize, $name), "</CENTER></TD>\n",
			      " </TR>\n";
		}
	}

	# Display row
	print " <TR>\n",
	      "  <TD>&nbsp;</TD>\n",
	      "  <TD>&nbsp;</TD>\n",
	      "  <TD><CENTER>", $cgi->submit($config->{'TxtSubmitButton'}), "</CENTER></TD>\n",
	      " </TR>\n";


	# End of table
	print "</TABLE>\n</CENTER>\n";
	
	# End query
	$statement->finish();


	# Write new information
	if (defined($param->{'FONTCTRL'}))
	{
		# Create entries
		$rcolor = join(':', @rcolor);
		$rface  = join(':', @rface);
		$rsize  = join(':', @rsize);


		# Update database
		$data->do("UPDATE Realm
			SET RealmFontColor=?,
			    RealmFontFace=?,
			    RealmFontSize=?
			WHERE RealmName=?", {},

		 $rcolor,
		 $rface,
		 $rsize,
		 $param->{'ROOM'});
	}
}

#####################
# Forcasts
# 
# Allows for the host to select a weather station from which the room can have
# weather based upon.   This is the four character ICAO indicator used for 
# METAR reports.

sub Forcasts
{
	####################
	# Data members

	my $database = shift;					# Database handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $table    = shift;					# Table handle
	my $tmpl     = shift;					# Template handler
	my $config   = shift;					# Config hash

	my $icoa;						# Station
	my $metar;						# Forcasts


	#####################
	# Program area

	# Retrieval
	($icoa, $metar) = $database->DataGet("SELECT
		  RealmICAO,
		  RealmMETAR
		FROM  Realm
		WHERE RealmName=?", $param->{'ROOM'});

	# Make sure they are defined
	$icoa  = (defined($icoa))  ? $icoa  : 'N/A';
	$metar = (defined($metar)) ? $metar : 'N/A';


	# HTML Display
	# Title
	$tmpl->ShowMain($config->{'TagForcastTitle'});

	# Star form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}), "\n";

	# Table top
	$table->MakeTop();

	# Forcast
	$table->MakeSingle($tmpl->PassSub($config->{'TagForcastData'}));
	$table->MakeSingle($metar);
	$table->MakeBlank();

	# Selected station
	$table->MakeSingle($tmpl->PassSub($config->{'TagForcastStation'}));
	$table->MakeSingle($cgi->textfield('STATION', $icoa, 4, 4));
	$table->MakeBlank();

	# Submit and bottom
	$table->MakeValid($cgi->submit($config->{'TxtSubmitAlter'}), $cgi->reset($config->{'TxtReset'}));
	$table->MakeBottom();

	# End of form
	print $cgi->end_form();	
}

#####################
# Locale Handler
#
# Handles the addition and removal of the locale settings for realms.  This
# will only apply if the realm actually selected strict mode.

sub LocaleHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $table    = shift;					# Table handler
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash

	my $locales;						# Locales widget
	my @locales;						# List of locales


	#####################
	# Program area
	
	# Fetch locale setting
	($verdict) = $database->DataGetRealmLocale($param->{'ROOM'});


	# Allow display
	if ($verdict eq 'strict')
	{
		# Fetch locales
		$database->GetListRealmLocale(\@locales, $param->{'ROOM'});

		# Append safety
		unshift(@locales, $config->{'TxtNone'});


		# Generate widget
		$locales = $cgi->scrolling_list('LOCLREM',  \@locales, $locales[0], 10);


		# HTML
		# Title
		$tmpl->ShowMain($config->{'TagLocaleTitle'});

		# Beginning of form
		print $cgi->start_form();
		print $param->EmbedNormal($cgi, %{$param}, 'LOCALE', 'True'), "\n";

		# Table
		$table->MakeTop();

		# Removal
		$table->MakeSingle($tmpl->PassSub($config->{'TagLocaleRem'}));
		$table->MakeValid($locales, $cgi->submit($config->{'TxtSubmitDelete'}), 'middle');
		$table->MakeBlank();

		# Addition
		$table->MakeSingle($tmpl->PassSub($config->{'TagLocaleAddition'}));
		$table->MakeValid($cgi->textfield('LOCLADD', '', 50, 255), $cgi->submit($config->{'TxtSubmitAddition'}));
		$table->MakeBottom();

		# End of form
		print $cgi->end_form(), "\n";
	}

	# Display notive
	else
	{
		$table->MakeTop();
		$table->MakeSingle($config->{'MsgLocale'});
		$table->MakeBottom();
	}
}


#####################
# Realm Select
#
# Basic display and prompt user for a selection on which realm to adminster and
# make modifications of some type to. 

sub RealmSelect
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $table    = shift;					# Table handle
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash

	my @realms;						# Array of realms
	my $realms;						# Popup menu


	#####################
	# Program area

	# Prepare of drop down
	$database->GetListSupervisorRealms(\@realms, $param->{'USER'});
	$realms = $cgi->popup_menu('ROOM', \@realms);

	# HTML Display
	# Title
	$tmpl->ShowMain($config->{'TagRealmSelect'});

	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}), "\n";

	# Table top
	$table->MakeTop();

	# Relam selection
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagRealmSelectRealm'}),
	 $realms);
	$table->MakeBlank();

	# Submit and bottom
	$table->MakeSingle($cgi->submit($config->{'TxtSubmitView'}));
	$table->MakeBottom();

	# End of form and embed
	print $cgi->end_form(), "\n";	
}


#####################
# Room Ban
#
# Controls the bannings of certain individuals, this is realm specific.   As a
# result a complete ban can result in a password change that will cause the 
# user to not be able to log on any furthur

sub RoomBan
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $table    = shift;					# Table offer
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $statement;						# Query statement

	my $date     = new Ethereal::Date();			# Date handler

	my $bird;						# Singular bird
	my $parole;						# Time till parole
	my $jailbirds;						# Jailbird widget

	my @jailbirds;						# List of jailbird puppets


	#####################
	# program area

	# Retreive list of banned and suspects
	# Prepare
	$statement = $data->prepare("SELECT
		 PuppeteerLogin,
		 RestrictionTimestamp
		FROM  PuppeteerRestriction
		WHERE RealmName=?");

	# Execute
	$statement->execute($param->{'ROOM'});
	

	# Retreive a list of all banned puppets
	while (($bird, $parole) = $statement->fetchrow())
	{ 
		# Human readable date
		$parole = $date->GetDateShort($parole);

		# Populate array
		push(@jailbirds, "$bird ($parole)");	
	}

	# Finish query
	$statement->finish();


	# Addition
	unshift(@jailbirds, $config->{'TxtNone'});

	# Format lists
	$jailbirds = $cgi->scrolling_list('BANNED',  \@jailbirds, $jailbirds[0], 10);


	# HTML
	$tmpl->ShowMain($config->{'TagBanTitle'});

	# Beginning of form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'BAN', 'true'), "\n";

	$table->MakeTop();
	$table->MakeSingle($tmpl->PassSub($config->{'TagBanRem'}));
	$table->MakeValid($jailbirds, $cgi->submit($config->{'TxtSubmitDelete'}), 'middle');
	$table->MakeBlank();
	$table->MakeBlank();

	$table->MakeSingle($tmpl->PassSub($config->{'TagBanAdd'}));
	$table->MakeValid(
	 $tmpl->Pass('TmplSub',  'MTITLE', $config->{'TagBanAddPuppet'}),
	 $cgi->textfield('SUSPECT', '', 20, 30));
	$table->MakeValid(
	 $tmpl->Pass('TmplSub',  'MTITLE', $config->{'TagBanAddTime'}),
	 $cgi->textfield('PAROLE', 0, 3, 3));

	$table->MakeBlank();
	$table->MakeSingle($tmpl->PassTitle($config->{'TagBanReason'}));
	$table->MakeSingle($cgi->textarea(-name=>'DREASON',-rows=>3,-columns=>50,-wrap=>'OFF',-default=>''));

	$table->MakeSingle($cgi->submit($config->{'TxtSubmitAddition'}));
	$table->MakeBottom();

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# Room Canon
#
# Allows for the host to add in canon handles, meaning handles that
# are restricted to the realm specifically and probably already exist
# on the system.

sub RoomCanon
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $table    = shift;					# Table handle
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash

	my $audit;						# Auditing
	my $data     = $database->{'HANDLE'};			# Database handle
	my $statement;						# Database statement

	my $name;						# Massaged name
	my $canon;						# Hash of cannon handles
	my %realm;						# Hash for realm tags


	#####################
	# Program area

	# Populate hash
	$database->GetHashRealm(\%realm);

	# Define instance
	$audit = new Ethereal::Audit($database, $cgi, $param);


	# Display canon handles
	# Prepare and execute
	$statement = $data->prepare("SELECT
		 PuppetName,
		 PuppeteerLogin
		FROM  Puppet
		WHERE PuppetPool=?
		ORDER BY PuppetName");
	$statement->execute($param->{'ROOM'});

	# Title
	$tmpl->ShowMain($config->{'TagCanonTitle'});
	


	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param});

	# Table top
	$table->MakeTop();
	$table->MakeSingle($tmpl->PassTitle($config->{'TagCanonList'}));
	$table->MakeBlank();

	# Cycle through
	while ($canon = $statement->fetchrow_hashref())
	{
		# Determin names
		$name =  $canon->{PuppetName};
		$name =~ s/\W//gs;


		# Hide
		if (defined($param->{$name}))
		{
		 	# Remove canon handle
			$data->do("DELETE FROM Puppet
				WHERE PuppetName=?
				  AND PuppetPool=?", {},

			 $canon->{'PuppetName'},
			 $param->{'ROOM'});	


			# Cleanup
			$param->Cleanup($name, $config->{'TxtSubmitDelete'});

			# Record in audit
			$audit->CanonDel($param->{'ROOM'}, $canon->{'PuppetName'});
		} 	

		# Display
		else
		{
			# Standard table
			$table->MakeValid("<B>$canon->{PuppetName}</B> ($canon->{PuppeteerLogin})", $cgi->checkbox(-name=>$name, -label=>''));
		}
	}

	# Kill button
	$table->MakeValid('&nbsp;', $cgi->submit($config->{'TxtSubmitDelete'}));

	# End of form
	print $cgi->end_form();

	# End query
	$statement->finish();


	# Spacer
	$table->MakeBlank();
	$table->MakeBlank();
	$table->MakeBlank();


	# Puppet Addition
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param});


	# HTML Display
	# Table header
	$table->MakeSingle($tmpl->PassTitle($config->{'TagCanonAddition'}));
	$table->MakeBlank();

	
	# Widgets
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagCanonPuppet'}),
	 $cgi->textfield('CCHAR', '', 15, 30));	
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagCanonPuppeteer'}),
	 $cgi->textfield('CUSER', '', 15, 10));	
	$table->MakeBlank();
	

	# Table bottom
	$table->MakeValid('&nbsp;', $cgi->submit($config->{'TxtSubmitAddition'}));
	$table->MakeBottom();
	

	# End of form
	print $cgi->end_form();
}



#####################
# Room Design
#
# Simply put, this section will be handling the design aspects of a realm
# this deals with the splash page, the header and footer used o the pages
# as well as modifications to the communications panel.

sub RoomDesign
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $table    = shift;					# Table handle
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash

	my $fields;						# Useable format
	my $open;						# Open document
	my $text;						# Text area text

	my @fields;						# Fields to edit


	#####################
	# Program area

	# Fetch design
	$database->GetListDesign(\@fields);


	# Assign value
	$open = (defined($param->{'DFILD'})) ? $param->{'DFILD'} : 'RealmInfo';

	# Generate widget
	$fields = $cgi->popup_menu('DFILD', \@fields, $open); 


	# Fetch  Document
	$text = $database->FetchRealm($open, $param->{'ROOM'});

	# Cleanup
	$param->Cleanup('DFILD', 'DTEXT');


	# HTML display
	# Title
	$tmpl->ShowMain($config->{'TagDesignTitle'});

	# Document selection
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'DESIGN', 'True'), "\n";	

	# Seperator bar
	$table->MakeTop();
	$table->MakeSingle($tmpl->PassSub($config->{'TagDesignField'}));
	$table->MakeValid($fields, $cgi->submit($config->{'TxtSubmitView'}));
	$table->MakeBlank();
	$table->MakeBlank();


	# Text modification
	# End of form 
	print $cgi->end_form(), "\n";

	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'DFILD', $open, 'DESIGN', 'True'), "\n";	

	# Replace macro
	$config->{TagDesignText} =~ s/MDOCUMENT/$open/gs;

	# Seperator bar
	$table->MakeSingle($tmpl->PassSub($config->{TagDesignText}));
	$table->MakeSingle($cgi->textarea(-name=>'DTEXT',-rows=>20,-columns=>70,-wrap=>'OFF',-default=>$text));
	$table->MakeBlank();

	# Submit and bottom
	$table->MakeValid($cgi->submit($config->{'TxtSubmitButton'}), $cgi->submit($config->{'TxtResetButton'}));
	$table->MakeBottom();

	# End of form 
	print $cgi->end_form(), "\n";
}


#####################
# Room Moderators
#
# Allows for the addition of moderators into the system itself.  While not 
# overly necessary as no power is actually gotten from this.  It will
# allow to see more trusted people within the realms.

sub RoomMods
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $table    = shift;					# Table handler
	my $tmpl     = shift;					# Template handler
	my $config   = shift;					# Config hash

	my $moderators;						# List of moderator puppets
	my @moderators;						# List of moderator puppets


	#####################
	# program area

	# Retreive list of moderators
	$database->GetList(\@moderators, "SELECT p.PuppeteerLogin
		FROM  Puppeteer p, RealmSupervisor r
		WHERE p.PuppeteerLogin = r.PuppeteerLogin
		  AND p.PuppeteerModPrivs='yes'
		  AND r.RealmName=?", 
	 $param->{'ROOM'});

	# Append safety measure
	unshift(@moderators, $config->{'TxtNone'});

	# Format widget
	$moderators = $cgi->scrolling_list('MODREM',  \@moderators, $moderators[0], 10);


	# HTML
	# Title
	$tmpl->ShowMain($config->{'TagModTitle'});

	# Beginning of form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'MOD', 'true'), "\n";

	# Table top
	$table->MakeTop();

	# Remove
	$table->MakeSingle($tmpl->PassSub($config->{'TagModRem'}));
	$table->MakeValid($moderators, $cgi->submit($config->{'TxtSubmitDelete'}), 'middle');
	$table->MakeBlank();

	# Addition
	$table->MakeSingle($tmpl->PassSub($config->{'TagModAdd'}));
	$table->MakeValid($cgi->textfield('MODADD', '', 20, 30), $cgi->submit($config->{'TxtSubmitAddition'}));
	$table->MakeBottom();

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# Room Picture Ban
#
# This manages the basic display for the addition and removal of pictures from
# the banned list.  Banned implies that the image cannot be used again on the
# system.

sub RoomPicture
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $table    = shift;					# Table handle
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash

	my $pics;						# List of pics
	my @pics;						# List of pics


	#####################
	# program area

	# Retreive list of images
	$database->GetListImageBan(\@pics);
	unshift(@pics, $config->{'TxtNone'});

	# Create widget
	$pics = $cgi->scrolling_list('PICREM', \@pics, $pics[0], 10);


	# HTML
	# Title
	$tmpl->ShowMain($config->{'TagPicTitle'});

	# Beginning of form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'GRAPHS', 'True'), "\n";

	# Table
	$table->MakeTop();

	# Removal
	$table->MakeSingle($tmpl->PassSub($config->{'TagPicRem'}));
	$table->MakeValid($pics, $cgi->submit($config->{'TxtSubmitDelete'}), 'middle');
	$table->MakeBlank();

	# Addition
	$table->MakeSingle($tmpl->PassSub($config->{'TagPicAdd'}));
	$table->MakeValid($cgi->textfield('PICADD', '', 50, 255), $cgi->submit($config->{'TxtSubmitAddition'}));
	$table->MakeBottom();

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# RoomView
# 
# Will allow the supervisors to view the selected room and make changes as necessary
# and will probably make the changes themselves.

sub RoomView
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter hash
	my $table    = shift;					# Table handle
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash
	my $admin    = shift;					# Administrative access
 
	my $comm;						# Comm panel handle
	my $data     = $database->{'HANDLE'};			# Database handle
	my $option;						# Option handler
	my $statement;						# Database statement

	my $access;						# Access control
	my $banner;						# Banner widget
	my $color;						# Colour selection default
	my $dateloc;						# Location of timestamp
	my $lang;						# Language restriction
	my $locale;						# Locale display
	my $genre;						# Realm genre
	my $guests;						# Guests access
	my $rating;						# Realm rating
	my $realm;						# Realm information
	my $tags;						# Tag enforcement
	my $verified;						# Verification enforcement

	my @optcolr;						# Colour choices 
	my @optgnre;						# Genre choices
	my @optrate;						# Rating choices

	my @optaccs = qw(Relaxed Restricted List);		# Access control
	my @optbner = qw(Display Exempt);			# Banner display
	my @optdate = qw(Bottom Top Removed);			# Timestamp location
	my @optlang = qw(Relaxed Restricted);			# Language access
	my @optlocl = qw(None Strict Variable);			# Locale display
	my @optgust = qw(Yes No Post);				# Guest access
	my @opttags = qw(Relaxed Restricted);			# Tag enforcement
	my @optvery = qw(Yes No);				# Tag enforcement


	#####################
	# Program area

	# Create instances
	$comm   = new Ethereal::Comm($database, $cgi);
	$option = new Ethereal::Option();

	# Pull colours
	@optcolr = $comm->FontSelectColor($param);

	# Pull genre and rating
	$option->Split(\@optgnre, $config->{'SetGenre'});
	$option->Split(\@optrate, $config->{'SetRating'});


	# Populate realm information hashref
	# Prepare and execute
	$statement = $data->prepare("SELECT RealmName,
		 RealmDescription,
		 RealmGenre,
		 RealmRating,
		 RealmLanguage, 
		 RealmFilter,
		 RealmAccess, 
		 RealmVerified, 
		 RealmGuests, 
		 RealmTags, 
		 RealmLocale, 
		 RealmBanner, 
		 RealmTextColour,
		 RealmFramesetRows,
		 RealmFramesetCols,
		 RealmImageHeight, 
		 RealmImageWidth,
		 RealmImageSize,
		 RealmDateLoc,
		 RealmAltBoard,
		 RealmArchive,
		 RealmBoard
		FROM Realm 
		WHERE RealmName=?");
	$statement->execute($param->{'ROOM'});

	# Retreive and finish
	$realm = $statement->fetchrow_hashref();
	$statement->finish();

	# Prepare radiobuttons
	$access   = $cgi->radio_group('ACCESS', \@optaccs, ucfirst($realm->{'RealmAccess'}), 'true');
	$banner   = $cgi->radio_group('BANNER', \@optbner, ucfirst($realm->{'RealmBanner'}), 'true');
	$dateloc  = $cgi->radio_group('DATLOC', \@optdate, ucfirst($realm->{'RealmDateLoc'}), 'true');
	$lang     = $cgi->radio_group('LANG',   \@optlang, ucfirst($realm->{'RealmLanguage'}), 'true');
	$locale   = $cgi->radio_group('LOCL',   \@optlocl, ucfirst($realm->{'RealmLocale'}), 'true');
	$guests   = $cgi->radio_group('GUEST',  \@optgust, ucfirst($realm->{'RealmGuests'}), 'true');
	$tags     = $cgi->radio_group('TAGS',   \@opttags, ucfirst($realm->{'RealmTags'}), 'true');
	$verified = $cgi->radio_group('VERIFY', \@optvery, ucfirst($realm->{'RealmVerified'}), 'true');

	# Different widget
	$color    = $cgi->popup_menu('COLOR',  \@optcolr, $realm->{'RealmTextColour'});
	$genre    = $cgi->popup_menu('GENRE',  \@optgnre, $realm->{'RealmGenre'});
	$rating   = $cgi->popup_menu('RATING', \@optrate, $realm->{'RealmRating'});

        # Assign values
        $realm->{'RealmAltBoard'}    = (defined($realm->{'RealmAltBoard'}))    ? $realm->{'RealmAltBoard'} : 'http://';
        $realm->{'RealmBoard'}       = (defined($realm->{'RealmBoard'}))       ? $realm->{'RealmBoard'} : '';
	$realm->{'RealmArchive'}     = (defined($realm->{'RealmArchive'}))     ? $realm->{'RealmArchive'} : '';
	$realm->{'RealmDescription'} = (defined($realm->{'RealmDescription'})) ? $realm->{'RealmDescription'} : '';
	$realm->{'RealmFilter'}      = (defined($realm->{'RealmFilter'}))      ? $realm->{'RealmFilter'} : '****';


	# HTML display
	# Title
	$tmpl->ShowMain($param->{'ROOM'});

	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'MAIN', 'True'), "\n";	

	# Table top
	$table->MakeTop();
	
	# Access control
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagRealmAccess'}),
	 $access);
	$table->MakeBlank();

	# Locale handling
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagRealmLocale'}),
	 $locale);
	$table->MakeBlank();

	# Guest access
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagRealmGuests'}),
	 $guests);
	$table->MakeBlank();

	# Realm tag control
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagRealmTags'}),
	 $tags);
	$table->MakeBlank();

	# Date locale
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagRealmDateLoc'}),
	 $dateloc);
	$table->MakeBlank();


	# Administrative
	# Banner exemption
	if ($admin eq 'yes')
	{
		$table->MakeValid(
		 $tmpl->PassSub($config->{'TagRealmBanner'}),
		 $banner);
		$table->MakeBlank();

		$table->MakeValid(
		 $tmpl->PassSub($config->{'TagRealmVerified'}),
		 $verified);
		$table->MakeBlank();
	}

	# Spacer 
	$table->MakeBlank();

	# Language
	# Realm Language
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagRealmLanguage'}),
	 $lang);
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagRealmFilter'}),
	 $cgi->textfield('FILTER', $realm->{'RealmFilter'}, 10, 10));
	$table->MakeBlank();
	$table->MakeBlank();

	# Default font
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagRealmFontDef'}),
	 $color);
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagRealmGenre'}),
	 $genre);
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagRealmRating'}),
	 $rating);
	$table->MakeBlank();
	$table->MakeBlank();

	# Framesets
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagFrameRow'}),
	 $cgi->textfield('FRAMEROW', $realm->{'RealmFramesetRows'}, 3, 3));
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagFrameCol'}),
	 $cgi->textfield('FRAMECOL', $realm->{'RealmFramesetCols'}, 3, 3));
	$table->MakeBlank();
	$table->MakeBlank();


	# Image restriction
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagPupImgHeight'}),
	 $cgi->textfield('IMGHEIGHT', $realm->{'RealmImageHeight'}, 3, 3));
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagPupImgWidth'}),
 	 $cgi->textfield('IMGWIDTH',  $realm->{'RealmImageWidth'}, 3, 3));
	$table->MakeValid(
	 $tmpl->PassSub($config->{'TagPupImgSize'}),
 	 $cgi->textfield('IMGSIZE',  $realm->{'RealmImageSize'}, 3, 3));
	$table->MakeBlank();
	$table->MakeBlank();

	# Realm description
	$table->MakeSingle($tmpl->PassTitle($config->{'TagRealmDescription'}));
	$table->MakeSingle($cgi->textarea(-name=>'RDESCR',-rows=>3,-columns=>51,-wrap=>'OFF',-default=>$realm->{'RealmDescription'}));
	$table->MakeBlank();

	# Archive Email address
	$table->MakeSingle($tmpl->PassTitle($config->{'TagRealmArchive'}));
	$table->MakeSingle($cgi->textfield('ARCHIVE', $realm->{'RealmArchive'}, 70, 45));
	$table->MakeBlank();

	# Board administrative address
	$table->MakeSingle($tmpl->PassTitle($config->{'TagRealmBoard'}));
	$table->MakeSingle($cgi->textfield('BOARDS', $realm->{'RealmBoard'}, 70, 45));
	$table->MakeBlank();

	# Alternate message board
	$table->MakeSingle($tmpl->PassTitle($config->{'TagRealmAltBoard'}));
	$table->MakeSingle($cgi->textfield('ALTBOARD', $realm->{'RealmAltBoard'}, 70, 255));
	$table->MakeBlank();

	# Submit and bottom
	$table->MakeValid($cgi->submit($config->{'TxtSubmitButton'}), $cgi->submit($config->{'TxtResetButton'}));
	$table->MakeBottom();

	# End of form 
	print $cgi->end_form(), "\n";
}


#####################
# Room Word
#
# Controls word restrictions, as a result one can add or remove words for which
# such words will be replaced from posts by ****.  Simplistic but effective.

sub RoomWord
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $table    = shift;					# Table handle
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Config hash

	my $words;						# List of words
	my @words;						# List of words	


	#####################
	# program area

	# Retreive list of banned and suspects
	$database->GetListWords(\@words, $param->{'ROOM'});
	unshift(@words, $config->{'TxtNone'});

	# Create widget
	$words = $cgi->scrolling_list('WORDREM',  \@words, $words[0], 10);


	# HTML
	# Title
	$tmpl->ShowMain($config->{'TagWordTitle'});

	# Beginning of form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'WORDS', 'True'), "\n";

	# Table Top
	$table->MakeTop();

	# Remove
	$table->MakeSingle($tmpl->PassSub($config->{'TagWordRem'}));
	$table->MakeValid($words, $cgi->submit($config->{'TxtSubmitDelete'}), 'middle');
	$table->MakeBlank();

	# Addition
	$table->MakeSingle($tmpl->PassSub($config->{'TagWordAdd'}));
	$table->MakeValid($cgi->textfield('WORDADD', '', 10, 10), $cgi->submit($config->{'TxtSubmitAddition'}));
	$table->MakeBottom();

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# Room Write
# 
# Will handle the writing of information gathered from the forms. 

sub RoomWrite
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Config hash
	my $access   = shift;					# Access realms
	my $admin    = shift;					# Administrative access

	my $audit;						# Audit handler
	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $date     = new Ethereal::Date();			# Date handle
	my $option;						# Option handle
	my $send;						# Mail sending

	my $choice;						# Choice of the matter
	my $count;						# Counter
	my $time;						# Time values

	my $puppet;						# Puppet name
	my $puppeteer;						# Puppeteers name

	my @gummies;						# Gummies

	my %system;						# System hash


	#####################
	# Program area
	
	# Audit handle
	$audit       = new Ethereal::Audit($database, $cgi, $param);
	
	# Word restrictions
	if (defined($param->{'WORDS'}))
	{
		# Removal
		if ((defined($param->{"$config->{'TxtSubmitDelete'}"}))
		 && ($param->{'WORDREM'} ne $config->{'TxtNone'}))
		{ 
			$data->do("DELETE FROM Restriction 
				WHERE RealmName=?
				  AND RestrictionType=?
				  AND RestrictionValue=?", {}, 
			 $param->{'ROOM'},
			 'word', 
			 $param->{'WORDREM'});

			# Audit entry
			$audit->WordRem($param->{'ROOM'}, $param->{'WORDREM'});
		}

		# Addition
		elsif (defined($param->{"$config->{'TxtSubmitAddition'}"}))
		{ 
			# Only if a word
			if (($param->{'WORDADD'} !~ /^\W+/sg)
			 && (length($param->{'WORDADD'}) > 2))
			{
				$data->do("INSERT INTO Restriction 
					(RealmName,
					 RestrictionType,
					 RestrictionValue)
					VALUES (?,?,?)", {}, 

			 	 $param->{'ROOM'},
			 	 'word', 
			 	 $param->{'WORDADD'});

				# Audit entry
				$audit->WordAdd($param->{'ROOM'}, $param->{'WORDADD'});
			}
		}	


		# Cleanup
		$param->Cleanup(
			'WORDS',
			$config->{'TxtSubmitDelete'},
			$config->{'TxtSubmitAddition'}
		 );
	}


	# Board Topics
	elsif (defined($param->{'TOPICS'}))
	{
		# Removal of posts and topic
		if ((defined($param->{"$config->{'TxtSubmitDelete'}"}))
		 && ($param->{'TOPICREM'} ne $config->{'TxtDefault'}))
		{
			# Remove associated posts
			$data->do("DELETE FROM Board 
				WHERE RealmName=?
				  AND BoardTopic=?", {},
			 $param->{'ROOM'},
			 $param->{'TOPICREM'});

			# Remove the topic
			$data->do("DELETE FROM BoardTopic 
				WHERE RealmName=?
				  AND BoardTopic=?", {}, 
			 $param->{'ROOM'},
			 $param->{'TOPICREM'});

			# Audit entry
			$audit->TopicAdd($param->{'ROOM'}, $param->{'TOPICREM'});
		}

		# Addition of topics
		elsif ((defined($param->{"$config->{'TxtSubmitAddition'}"}))
		 && ($param->{'TOPICADD'} !~ /^system/i))
		{
			# Verify validity
			if (($param->{'TOPICADD'} !~ /^\W+/sg)
			 && (length($param->{'TOPICADD'}) > 2))
			{
				$data->do("INSERT INTO BoardTopic 
					(BoardTopic,
					 RealmName)
					VALUES (?,?)", {}, 

			  	 $param->{'TOPICADD'},
			 	 $param->{'ROOM'});

				# Audit entry
				$audit->TopicAdd($param->{'ROOM'}, $param->{'TOPICADD'});
			}
		}	


		# Cleanup
		$param->Cleanup(
			'TOPICS',
			$config->{'TxtSubmitDelete'},
			$config->{'TxtSubmitAddition'}
		 );
	}

	
	# Picture restrictions
	elsif (defined($param->{'GRAPHS'}))
	{
		# Removal
		if ((defined($param->{"$config->{'TxtSubmitDelete'}"})) && ($param->{'GRAPH'} ne $config->{'TxtNone'}))
		{ 
			$data->do("DELETE FROM Restriction 
				WHERE RealmName=?
				  AND RestrictionType=?
				  AND RestrictionValue=?", {}, 

			 'system',
			 'picture', 
			 $param->{'PICREM'});

			# Audit entry
			$audit->PicRem($param->{'ROOM'}, $param->{'PICREM'});
		}

		# Addition
		elsif (defined($param->{"$config->{'TxtSubmitAddition'}"}))
		{ 
			# Only if a word
			if (length($param->{'PICADD'}) > 10)
			{
				# Ban the image
				$data->do("INSERT INTO Restriction 
					(RealmName,
					 RestrictionType,
					 RestrictionValue)
					VALUES (?,?,?)", {}, 

			 	'system',
			 	'picture', 
			 	$param->{'PICADD'});

				# Clear all instances of image
				$data->do("UPDATE Puppet
					SET PuppetPic=NULL,
					    PuppetPicHeight=NULL,
					    PuppetPicWidth=NULL
					WHERE PuppetPic LIKE ?", {},

				"$param->{PICADD}%");

				# Audit entry
				$audit->PicAdd($param->{'ROOM'}, $param->{'PICADD'});
			}
		}	

		# Cleanup
		$param->Cleanup(
			'PICS',
			$config->{'TxtSubmitDelete'},
			$config->{'TxtSubmitAddition'}
		 );
	}

	# For banning handling
	elsif (defined($param->{'BAN'}))
	{
		# Removal of person
		if ((defined($param->{"$config->{'TxtSubmitDelete'}"})) && ($param->{'BANNED'} ne $config->{'TxtNone'}))
		{
			# Strip date information
			$param->{'BANNED'} =~ s/\s\(\d{4}\/\d{2}\/\d{2}\)//s;
			
			# Proper formatting
			($puppeteer) = $database->DataGetPuppeteerExistence($param->{'BANNED'});
			
			# Remove if he is a valid user
		 	if (defined($puppeteer))
			{	
				# Remove
				$data->do("DELETE FROM PuppeteerRestriction 
					WHERE PuppeteerLogin=?
					  AND RealmName=?", {}, 
				 $puppeteer,
				 $param->{'ROOM'});
	
				# Audit entry
				$audit->BanRem($param->{'ROOM'}, $puppeteer);
			}
		}

		# Addition
		elsif (defined($param->{"$config->{'TxtSubmitAddition'}"}))
		{
			# Determine existence
			($puppeteer) = $database->DataGetPuppeteerExistence($param->{'SUSPECT'});

			# Verification
			if ((length($param->{'SUSPECT'}) > 2) && ($param->{'SUSPECT'} !~ /^\s/) && (defined($puppeteer)))
			{
				# Make sure the timestamp is valid
				$param->{'PAROLE'} = ($param->{'PAROLE'} =~ /\d+/) ? $param->{'PAROLE'} : 0;
				$param->{'PAROLE'} = ($param->{'PAROLE'} < 1000)   ? $param->{'PAROLE'} : 1000;
				
				# Determine when one is paroled
				# End of Unix time
				if ($param->{'PAROLE'} < 1)
				{
					# Hard coded for simplicity
					$time = 2147483647;
				}

				# Less restrictive
				else
				{
					# Translate into proper Unix time
					$time = (time + (86400 * $param->{'PAROLE'}));
				}	


				# Write and record addition
				$data->do("INSERT INTO PuppeteerRestriction 
					(PuppeteerLogin,
					 RealmName,
					 RestrictionTimestamp)	
					VALUES (?,?,?)", {}, 
				 $puppeteer, 
				 $param->{'ROOM'}, 
				 $time);

				# Email notificiation
				if ($access ne 'list')
				{
					# Link with hash
					$database->GetHashSystem(\%system);

					# Pull values
					my ($banned) = $database->DocumentGetMailBanned();
					my ($email)  = $database->DataGetPuppeteerEmail($puppeteer);
					my ($name)   = $database->DataGetPuppeteerName($puppeteer);
					my ($parole) = $date->GetDateShort($time);

					# Default reason
					$param->{'DREASON'} = (length($param->{'DREASON'}) > 3) ? $param->{'DREASON'} : $system{'SetBannedDefault'};


					# Mail sending
					# Initiate handle
					$send = tie(*MAIL, 'Ethereal::Mail', $database);

					# Initial settings
					$send->SetSubject("You have been banned from $param->{ROOM}");
					$send->SetSearch(
						BANDATE    => $parole,
						BANREASON  => $param->{'DREASON'},
						FULLNAME   => $name,
						ROOMNAME   => $param->{'ROOM'},
						SERVERNAME => $system{'SetInfoServer'},
						USERNAME   => $puppeteer
					 );
					$send->ToggleWrap();

					# Recipients
					$send->AddTo("$name <$email>");
					$send->AddBCC("$system{SetBannedName} <$system{SetBannedAddress}>");


					# Message
					print MAIL $banned;

					# Close and send
					close(MAIL);
				}



				# Record in audit
				$audit->BanAdd($param->{'ROOM'}, $puppeteer);
			}
		}

		# Cleanup
		$param->Cleanup(
			'BAN',
			'TIMESPAN',
			$config->{'TxtSubmitDelete'},
			$config->{'TxtSubmitAddition'}
		 );
	}


	# Handling of canon handles
	elsif (defined($param->{'CANON'}))
	{
		# Addition
		if ((defined($param->{'CCHAR'})) 
		 && (defined($param->{'CUSER'})) 
		 && (length($param->{'CCHAR'}) > 2))
		{
			# Determine existence
			($puppeteer) = $database->DataGetPuppeteerExistence($param->{'CUSER'});
			($puppet)    = $database->DataGetPuppetExistence($param->{'CCHAR'}, $param->{'CUSER'});


			# Everything in place
			if ((defined($puppeteer))
			 && (!defined($puppet)))
			{
				# Find gummy default
				$option = new Ethereal::Option();
				$option->Split(\@gummies, $config->{'OptGummyCanonFiles'});


				# Create handle
				$data->do("INSERT INTO Puppet
					(PuppetName,
					 PuppeteerLogin,
					 PuppetPool,
					 PuppetGummy)
					VALUES(?,?,?,?)", {},

				 $param->{'CCHAR'},
				 $puppeteer,
				 $param->{'ROOM'},
				 $gummies[0]);


				# Audit entry
				$audit->CanonAdd($param->{'ROOM'}, $param->{'CCHAR'});
			}
		}


		# Cleanup
		$param->Cleanup(
		 'CDEL',
		 'CCHAR',
		 'CUSER'
		);
	}


	# Local handler
	elsif (defined($param->{'LOCALE'}))
	{
		# Locale additions
		if ((defined($param->{$config->{'TxtSubmitAddition'}}))
		 && (defined($param->{'LOCLADD'}))
		 && (length($param->{'LOCLADD'}) > 2))
		{
			# Insert locale
			$data->do("INSERT INTO RealmLocale
				(RealmLocale,
				 Realmname)
				VALUES (?,?)", {},

			 $param->{'LOCLADD'},
			 $param->{'ROOM'});

			# Auditing entry
			$audit->LocaleAdd($param->{'ROOM'}, $param->{'LOCLADD'});

			# Cleanup
			$param->Cleanup('LOCLADD', $config->{'TxtSubmitAddition'});
		}

		# Locale removals
		elsif  ((defined($param->{$config->{'TxtSubmitDelete'}}))
	 	 && ($param->{'LOCLADD'} ne $config->{'TxtNone'}))
		{
			# Remove locale
			$data->do("DELETE FROM RealmLocale
				WHERE RealmLocale=?
				  AND RealmName=?", {},

			 $param->{'LOCLREM'},
			 $param->{'ROOM'});

			# Auditing entry
			$audit->LocaleRem($param->{'ROOM'}, $param->{'LOCLREM'});

			# Cleanup
			$param->Cleanup('LOCLREM', $config->{'TxtSubmitDelete'});
		}
	}


	# Handling of design
	elsif ((defined($param->{'DESIGN'})) && (defined($param->{$config->{'TxtSubmitButton'}})))
	{
		# Determine Realm Information
		if (length($param->{'DTEXT'}) > 15)
		{
			# Verify for existence
			($choice) = $database->DataGet("SELECT RealmName
				FROM RealmDesign
				WHERE RealmName=?
				  AND WebchatTextName=?", 

			 $param->{'ROOM'},
			 $param->{'DFILD'});
			

			# Update
			if (defined($choice))
			{
				# Update information
				$data->do("UPDATE RealmDesign
					SET DesignContent=?
					WHERE RealmName=?
					  AND WebchatTextName=?", {},

				 $param->{'DTEXT'},
				 $param->{'ROOM'}, 
 				 $param->{'DFILD'});
			}

			# Create instance
			else
			{
				# Insert information
				$data->do("INSERT INTO RealmDesign
					(DesignContent,
					 WebchatTextName,
					 RealmName)
					VALUES (?,?,?)", {},

				 $param->{'DTEXT'},
				 $param->{'DFILD'},
				 $param->{'ROOM'});
			}
		}

		# Delete entry
		else
		{
			# Remove row and use defaults
			$data->do("DELETE FROM RealmDesign
				WHERE RealmName=?
				  AND WebchatTextName=?", {},

			 $param->{'ROOM'},
			 $param->{'DFILD'});
		}

		# Audit entry
		$audit->InfoChange($param->{'ROOM'});

		# Cleanup
		$param->Cleanup(
		 'DTEXT',
		 $config->{'TxtSubmitButton'}
		);
	}

	# For moderator handling
	elsif (defined($param->{'MOD'}))
	{
		# Removal of person
		if ((defined($param->{$config->{'TxtSubmitDelete'}})) && ($param->{'MODREM'} ne $config->{'TxtNone'}))
		{
			# Initial handle
			# Remove from realm
			$data->do("DELETE FROM RealmSupervisor
				WHERE PuppeteerLogin=?
				  AND RealmName=?", {},

			 $param->{'MODREM'},
			 $param->{'ROOM'});

			# Audit entry
			$audit->RoomModRem($param->{'MODREM'}, $param->{'ROOM'});


			# Retreive positions
			($count) = $database->DataGetModCount($param->{'MODREM'});

			# Moderator bit handling
			if ($count < 1)
			{
				# Update database
				$data->do("UPDATE Puppeteer 
					SET   PuppeteerModPrivs='no' 
					WHERE PuppeteerLogin=?", {}, 
				 $param->{'MODREM'});

				$data->do("UPDATE Puppet 
					SET   PuppetGummy=NULL 
					WHERE PuppeteerLogin=?", {}, 
				 $param->{'MODREM'});

				# Audit entry
				$audit->ModsRem($param->{'MODREM'}, $param->{'MODREM'});
			}
		}

		# Addition
		elsif (defined($param->{$config->{'TxtSubmitAddition'}}))
		{

			# Verification
			if ((length($param->{'MODADD'}) > 2) && ($param->{'MODADD'} !~ /^\s/))
			{
				# Retreive supervisory bit
				($choice) = $database->DataGetSuper($param->{'MODADD'});

				# Only if not a supervisor
				if ($choice ne 'yes')
				{
					# Retreive positions
					($count) = $database->DataGetModCount($param->{'MODADD'});


					# Change moderator bit
					if ($count < 1)
					{
						# Change host bit
						$data->do("Update Puppeteer 
							SET   PuppeteerModPrivs='yes' 
							WHERE PuppeteerLogin=?", {}, 
						 $param->{'MODADD'});

						# Record in audit
						$audit->ModsAdd($param->{'MODADD'}, $param->{'MODADD'});
					}
				
					# Add room specific entry
					$data->do("INSERT INTO RealmSupervisor
						(PuppeteerLogin,
						 RealmName)
						VALUES(?,?)", {},
 
					 $param->{'MODADD'},
					 $param->{'ROOM'});

					# Record in audit
					$audit->RoomModAdd($param->{'MODADD'}, $param->{'ROOM'});
				}
			}
		}

		# Cleanup
		$param->Cleanup(
		 'MOD',
		 $config->{'TxtSubmitDelete'},
		 $config->{'TxtSubmitAddition'}
		);
	}

	# Update main information
	elsif (defined($param->{'MAIN'}))
	{
		# Determine validity
		$param->{'ARCHIVE'}  = ($param->{'ARCHIVE'} =~ /^\w(\w|\.|\-)*?\@(\w|\-)*?\..*?/s) ? $param->{'ARCHIVE'} : undef;
		$param->{'BOARDS'}   = ($param->{'BOARDS'}  =~ /^\w(\w|\.|\-)*?\@(\w|\-)*?\..*?/s) ? $param->{'BOARDS'} : undef;
		$param->{'ALTBOARD'} = ((length($param->{'ALTBOARD'}) > 12) && ($param->{'ALTBOARD'} =~ /^http:\/\//is)) ? $param->{'ALTBOARD'} : undef;
		$param->{'FILTER'}   = (length($param->{'FILTER'}) > 1) ? $param->{'FILTER'} : undef;


		# Realm description
		$param->{'RDESCR'} = (length($param->{'RDESCR'}) > 50) ? $param->{'RDESCR'} : undef;

		# Massage description
		if (defined($param->{'RDESCR'}))
		{
			# Truncate
			$param->{'RDESCR'} = substr($param->{'RDESCR'}, 0, 250);
		
			# Replace quotes
			$param->{'RDESCR'} =~ s/'/`/gs;
			$param->{'RDESCR'} =~ s/\n/ /gs;
			$param->{'RDESCR'} =~ s/\r/ /gs;
		}


		# Banner exemption
		if ($admin eq 'yes')
		{
			# Special update
			$data->do("UPDATE Realm
				SET RealmBanner=?,
				    RealmVerified=?
				WHERE RealmName=?", {},

			 lc($param->{'BANNER'}),
			 lc($param->{'VERIFY'}),
			 $param->{'ROOM'});

			# Cleanup
			$param->Cleanup('BANNER');
		}


		# Ensure numeric
		$param->{'IMGHEIGHT'} = ($param->{'IMGHEIGHT'} =~ /^\d+$/) ? $param->{'IMGHEIGHT'} : 0;
		$param->{'IMGWIDTH'}  = ($param->{'IMGWIDTH'}  =~ /^\d+$/) ? $param->{'IMGWIDTH'}  : 0;
		$param->{'IMGSIZE'}   = ($param->{'IMGSIZE'}   =~ /^\d+$/) ? $param->{'IMGSIZE'}   : 0;

		# Defaults otherwise
		$param->{'FRAMEROW'}  = ($param->{'FRAMEROW'}  =~ /^\d+$/) ? $param->{'FRAMEROW'}  : undef;
		$param->{'FRAMECOL'}  = ($param->{'FRAMECOL'}  =~ /^\d+$/) ? $param->{'FRAMECOL'}  : undef;


		# Update primary information
		$data->do("UPDATE Realm 
			SET RealmDescription=?,
			    RealmGenre=?,
			    RealmRating=?,
			    RealmLanguage=?, 
			    RealmFilter=?, 
			    RealmAccess=?, 
			    RealmDateLoc=?,
			    RealmGuests=?, 
			    RealmTags=?,
			    RealmLocale=?,
			    RealmTextColour=?,
			    RealmFramesetRows=?,
			    RealmFramesetCols=?,
			    RealmImageHeight=?, 
			    RealmImageWidth=?,
			    RealmImageSize=?,
			    RealmArchive=?,
			    RealmBoard=?,
			    RealmAltBoard=?
			WHERE RealmName=?", {}, 

		 $param->{'RDESCR'},
		 $param->{'GENRE'},
		 $param->{'RATING'},
		 lc($param->{'LANG'}), 
		 $param->{'FILTER'},
		 lc($param->{'ACCESS'}), 
		 lc($param->{'DATLOC'}),
		 lc($param->{'GUEST'}), 
		 lc($param->{'TAGS'}),
		 lc($param->{'LOCL'}),
		 $param->{'COLOR'},
		 $param->{'FRAMEROW'},
		 $param->{'FRAMECOL'},
		 $param->{'IMGHEIGHT'}, 
		 $param->{'IMGWIDTH'},
		 $param->{'IMGSIZE'},
		 $param->{'ARCHIVE'},
		 $param->{'BOARDS'},
		 $param->{'ALTBOARD'},
		 $param->{'ROOM'});

		# Audit entry
		$audit->Realm($param->{'ROOM'});
			
		# Cleanup
		$param->Cleanup('MAIN');
	}

	# Weather forcasts
	elsif (defined($param->{'FORCAST'}))
	{
		if (defined($param->{$config->{'TxtSubmitAlter'}}))
		{
			# Data Members
			my $metar;
			my $station = $param->{'STATION'};


			# Assign values
			if ($station =~ /^\w{4}/)
			{
				# Convert to uppercase
				$station = uc($station);

				# Find Forcast
				my $forc = new Ethereal::Weather();
				($station, $metar) = $forc->Get($station);

			}
			else
			{
				# Assign to undefined
				$metar   = undef;
				$station = undef;
			}


			# Modification
			# Change database entry
			$data->do("UPDATE Realm
				SET RealmICAO=?,
				    RealmMETAR=?
				WHERE RealmName=?", {},

			 $station,
			 $metar,
			 $param->{'ROOM'});

			 # Record audit change
			 $audit->Forcast($param->{'ROOM'});	
		}

		# Cleanup
		$param->Cleanup($config->{'TxtSubmitAlter'});
	}
}
